@@ -6,17 +6,17 @@
 #' @param refit if FALSE, new data will be simulated and scaled residuals will be created by comparing observed data with new data. If TRUE, the model will be refit on the simulated data (parametric bootstrap), and scaled residuals will be created by comparing observed with refitted residuals.
 #' @param integerResponse if TRUE, noise will be added at to the residuals to maintain a uniform expectations for integer responses (such as Poisson or Binomial). Usually, the model will automatically detect the appropriate setting, so there is no need to adjust this setting.
 #' @param plot if TRUE, \code{\link{plotSimulatedResiduals}} will be directly run after the simulations have terminated
-#' @param ... parameters to pass to the simulate function of the model object. An important use of this is to specify whether simulations should be conditional on the current random effect estimates. See also details
-#' @param seed the random seed. The default setting, recommended for any type of data analysis, is to reset the random number generator each time the function is run, meaning that you will always get the same result when running the same code. NULL = no new seed is set, but previous random state will be restored after simulation. FALSE = no seed is set, and random state will not be restored. The latter two options are only recommended for simulation experiments. See vignette for details.
-#' @return An S3 class of type "DHARMa", essentially a list with various elements. Implemented S3 functions include plot, print and \code{\link{residuals.DHARMa}}. Residuals returns the calculated scaled residuals, which can also be accessed via $scaledResiduals. The returned object additionally contains an element 'scaledResidualsNormal', which contains the scaled residuals transformed to a normal distribution (for stability reasons not recommended)
+#' @param ... parameters to pass to the simulate function of the model object. An important use of this is to specify whether simulations should be conditional on the current random effect estimates, e.g. via re.form. Note that not all models support syntax to specify conditionao or unconditional simulations. See also details
+#' @param seed the random seed to be used within DHARMa. The default setting, recommended for most users, is keep the random seed on a fixed value 123. This means that you will always get the same randomization and thus teh same result when running the same code. NULL = no new seed is set, but previous random state will be restored after simulation. FALSE = no seed is set, and random state will not be restored. The latter two options are only recommended for simulation experiments. See vignette for details.
+#' @return An S3 class of type "DHARMa", essentially a list with various elements. Implemented S3 functions include plot, print and \code{\link{residuals.DHARMa}}. Residuals returns the calculated scaled residuals.
 #' 
 #' @details There are a number of important considerations when simulating from a more complex (hierarchical) model: 
 #' 
-#' \strong{Re-simulating random effects / hierarchical structure}: the first is that in a hierarchical model, several layers of stochasticity are aligned on top of each other. Specifically, in a GLMM, we have a lower level stochastic process (random effect), whose result enters into a higher level (e.g. Poisson distribution). For other hierarchical models such as state-space models, similar considerations apply. When simulating, we have to decide if we want to re-simulate all stochastic levels, or only a subset of those. For example, in a GLMM, it is common to only simulate the last stochastic level (e.g. Poisson) conditional on the fitted random effects. 
+#' \strong{Re-simulating random effects / hierarchical structure}: in a hierarchical model, we have several stochastic processes aligned on top of each other. Specifically, in a GLMM, we have a lower level stochastic process (random effect), whose result enters into a higher level (e.g. Poisson distribution). For other hierarchical models such as state-space models, similar considerations apply. 
 #' 
-#' For controlling how many levels should be re-simulated, the simulateResidual function allows to pass on parameters to the simulate function of the fitted model object. Please refer to the help of the different simulate functions (e.g. ?simulate.merMod) for details. For merMod (lme4) model objects, the relevant parameters are parameters are use.u and re.form
+#' In such a situation, we have to decide if we want to re-simulate all stochastic levels, or only a subset of those. For example, in a GLMM, it is common to only simulate the last stochastic level (e.g. Poisson) conditional on the fitted random effects. This is often referred to as a conditional simuation. For controlling how many levels should be re-simulated, the simulateResidual function allows to pass on parameters to the simulate function of the fitted model object. Please refer to the help of the different simulate functions (e.g. ?simulate.merMod) for details. For merMod (lme4) model objects, the relevant parameters are parameters are use.u and re.form
 #' 
-#' If the model is correctly specified, the simulated residuals should be flat regardless how many hierarchical levels we re-simulate. The most thorough procedure would therefore be to test all possible options. If testing only one option, I would recommend to re-simulate all levels, because this essentially tests the model structure as a whole. This is the default setting in the DHARMa package. A potential drawback is that re-simulating the lower-level random effects creates more variability, which may reduce power for detecting problems in the upper-level stochastic processes. 
+#' If the model is correctly specified, the simulated residuals should be flat regardless how many hierarchical levels we re-simulate. The most thorough procedure would therefore be to test all possible options. If testing only one option, I would recommend to re-simulate all levels, because this essentially tests the model structure as a whole. This is the default setting in the DHARMa package. A potential drawback is that re-simulating the lower-level random effects creates more variability, which may reduce power for detecting problems in the upper-level stochastic processes. In particular dispersion tests may produce different results when switching from conditional to unconditional simulations, and often the conditional simulation is more sensitive. 
 #' 
 #' \strong{Integer responses}: a second complication is the treatment of inter responses. Imaging we have observed a 0, and we predict 30\% zeros - what is the quantile that we should display for the residual? To deal with this problem and maintain a uniform response, the option integerResponse adds a uniform noise from -0.5 to 0.5 on the simulated and observed response, which creates a uniform distribution - you can see this via hist(ecdf(runif(10000))(runif(10000))).
 #' 
@@ -32,11 +32,11 @@
 #' 
 #' \strong{Residuals per group}: In many situations, it can be useful to look at residuals per group, e.g. to see how much the model over / underpredicts per plot, year or subject. To do this, use \code{\link{recalculateResiduals}}, together with a grouping variable (see also help)
 #' 
-#' \strong{Transformation to other distributions}: DHARMa calculates residuals for which the theoretical expectation (assuming a correctly specified model) is uniform. To transfor this residuals to another distribution (e.g. so that a correctly specified model will have normal residuals) see \code{\link{transformQuantiles}} 
+#' \strong{Transformation to other distributions}: DHARMa calculates residuals for which the theoretical expectation (assuming a correctly specified model) is uniform. To transfor this residuals to another distribution (e.g. so that a correctly specified model will have normal residuals) see \code{\link{residuals.DHARMa}}. 
 #' 
 #' @note See \code{\link{testResiduals}} for an overview of residual tests, \code{\link{plot.DHARMa}} for an overview of available plots. 
 #' 
-#' @seealso \code{\link{testResiduals}}, \code{\link{plot.DHARMa}}, \code{\link{print.DHARMa}}, \code{\link{residuals.DHARMa}}, \code{\link{recalculateResiduals}}, \code{\link{transformQuantiles}}
+#' @seealso \code{\link{testResiduals}}, \code{\link{plot.DHARMa}}, \code{\link{print.DHARMa}}, \code{\link{residuals.DHARMa}}, \code{\link{recalculateResiduals}}
 #' 
 #' @example inst/examples/simulateResidualsHelp.R
 #' @import stats
@@ -56,35 +56,22 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
   out = list()
   
   family = family(fittedModel)
-  if(is.null(integerResponse)){
-    if (family$family %in% c("binomial", "poisson", "quasibinomial", "quasipoisson", "Negative Binom", "nbinom2", "nbinom1", "genpois", "compois", "truncated_poisson", "truncated_nbinom2", "truncated_nbinom1", "betabinomial", "Poisson", "Tpoisson", "COMPoisson", "negbin", "Tnegbin") | grepl("Negative Binomial",family$family) ) integerResponse = TRUE
-    else integerResponse = FALSE
-  }
-  
   out$fittedModel = fittedModel
   out$modelClass = class(fittedModel)[1]
   
   out$nObs = nobs(fittedModel)
   out$nSim = n
   out$refit = refit
-  out$observedResponse = getResponse(fittedModel) 
+  out$observedResponse = getObservedResponse(fittedModel) 
   
-  # TODO - check if that works 
-  nKcase = is.matrix(out$observedResponse)
-  if(nKcase){
-    if(! (family$family %in% c("binomial", "betabinomial"))) securityAssertion("nKcase - wrong family")
-    if(! (ncol(out$observedResponse)==2)) securityAssertion("nKcase - wrong dimensions of response")
-    out$observedResponse = out$observedResponse[,1]
+  if(is.null(integerResponse)){
+    if (family$family %in% c("binomial", "poisson", "quasibinomial", "quasipoisson", "Negative Binom", "nbinom2", "nbinom1", "genpois", "compois", "truncated_poisson", "truncated_nbinom2", "truncated_nbinom1", "betabinomial", "Poisson", "Tpoisson", "COMPoisson", "negbin", "Tnegbin") | grepl("Negative Binomial",family$family) ) integerResponse = TRUE
+    else integerResponse = FALSE
   }
-
   out$integerResponse = integerResponse
+
   out$problems = list()
-  out$scaledResiduals = rep(NA, out$nObs)
 
-  ## following block re-used below, create function for this 
-
-  ##### calculating predictions #####
-  
   # re-form should be set to ~0 to avoid spurious residual patterns, see https://github.com/florianhartig/DHARMa/issues/43
     
   if(out$modelClass %in% c("HLfit")){
@@ -96,39 +83,14 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
   out$fittedFixedEffects = getFixedEffects(fittedModel)
   out$fittedResiduals = residuals(fittedModel, type = "response")
   
-  ######## simulations ##################
-  
-  simulations = getSimulations(fittedModel, nsim = n, ...)
-  
-  if(out$modelClass == "glmmTMB"){
-    if(is.vector(simulations[[1]])){
-      out$simulatedResponse = data.matrix(simulations)
-    } else if (is.matrix(simulations[[1]])){ 
-      # this is for the k/n binomial case
-      out$simulatedResponse = as.matrix(simulations)[,seq(1, (2*n), by = 2)]
-    } else securityAssertion("Simulation results produced unsupported data structure", stop = TRUE)
-    
-    # observation is factor - unlike lme4 and older, glmmTMB simulates nevertheless as numeric
-    if(is.factor(out$observedResponse)) out$observedResponse = as.numeric(out$observedResponse) - 1
-  }else{
-    if(is.vector(simulations[[1]])){
-      out$simulatedResponse = data.matrix(simulations)
-    } else if (is.matrix(simulations[[1]])){ 
-      # this is for the k/n binomial case
-      out$simulatedResponse = as.matrix(simulations)[,seq(1, (2*n), by = 2)]
-    } else if(is.factor(simulations[[1]])){
-      if(nlevels(simulations[[1]]) != 2) warning("The fitted model has a factorial response with number of levels not equal to 2 - there is currently no sensible application in DHARMa that would lead to this situation. Likely, you are trying something that doesn't work.")
-      out$simulatedResponse = data.matrix(simulations) - 1
-      out$observedResponse = as.numeric(out$observedResponse) - 1
-    } else securityAssertion("Simulation results produced unsupported data structure", stop = TRUE)
-  }
-    
-  if(any(dim(out$simulatedResponse) != c(out$nObs, out$nSim) )) securityAssertion("Simulation results have wrong dimension", stop = T)
-  
   ######## refit = F ################## 
 
   if (refit == FALSE){
     
+    out$simulatedResponse = getSimulations(fittedModel, nsim = n, type = "normal", ...)
+    
+    checkSimulations(out$simulatedResponse, out$nObs, out$nSim)
+    
     out$scaledResiduals = getQuantile(simulations = out$simulatedResponse , observed = out$observedResponse , n = out$nObs, nSim = out$nSim, integerResponse = integerResponse)
 
   ######## refit = T ################## 
@@ -142,19 +104,19 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
     out$refittedResiduals = matrix(nrow = out$nObs, ncol = n)   
     out$refittedPearsonResiduals = matrix(nrow = out$nObs, ncol = n)   
     
+    out$simulatedResponse = getSimulations(fittedModel, nsim = n, type = "refit", ...)
+    
     for (i in 1:n){
-      #tryCatch()
-      
-      if (out$modelClass == "glmmTMB" & ncol(simulations) == 2*n) simObserved = simulations[,(1+(2*(i-1))):(2+(2*(i-1)))]
-      else simObserved = simulations[[i]]
-      
+
+      simObserved = out$simulatedResponse[[i]]
+  
       try({
         
         # for testing
         # if (i==3) stop("x")
         # Note: also set silent = T for production
     
-        refittedModel = refit(fittedModel, simObserved)
+        refittedModel = getRefit(fittedModel, simObserved)
         
         out$refittedPredictedResponse[,i] = predict(refittedModel, type = "response")
         out$refittedFixedEffects[,i] = getFixedEffects(refittedModel)
@@ -186,8 +148,6 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
   }
 
   ########### Wrapup ############
-  
-  out$scaledResidualsNormal = qnorm(out$scaledResiduals + 0.00 )
 
   out$time = proc.time() - ptm
   out$randomState = randomState
@@ -201,8 +161,33 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
 
 getPossibleModels<-function()c("lm", "glm", "negbin", "lmerMod", "glmerMod", "gam", "bam", "glmmTMB", "HLfit") 
 
-checkModel <- function(fittedModel){
-  if(!(class(fittedModel)[1] %in% getPossibleModels())) warning("DHARMa: fittedModel not in class of supported models. Absolutely no guarantee that this will work!")
+#' Check if the fitted model is supported by DHARMa
+#' 
+#' The function checks if the fitted model is supported by DHARMa, and if there are other issues, for example the use of weights, that could create problems for calculating quantile residuals
+#' 
+#' @param fittedModel a fitted model 
+#' @param stop whether to throw an error if the model is not supported by DHARMa  
+#' 
+#' @details The main purpose of this function os to check if the fitted model class is supported by DHARMa. The function additionally checks for properties of the fitted model that could create problems for calculating residuals or working with the resuls in DHARMa. At the moment, two such issues are checked
+#' 
+#' 1) Use of weights: most regression models in R support the use of the weights argument. Unfortunately, the argument means different things, depending on the context. In many situations, weights basically reweights the likelihood. In this case, simulated quantile residuals cannot be used, because the weighting
+#' 
+#' 2) NA values in the data: checkModel will detect if there were NA values in the data frame. For NA values, most regression models will remove the entire observation from the data. This is not a problem for DHARMa - residuals are then only calculated for non-NA rows in the data. However, if you provide additional predictors to DHARMa, for example to plot residuals against a predictor, you will have to remove all NA rows that were also removed in the model, otherwise the lengths of the vectors will not match. 
+#' 
+#' 
+#' @keywords internal
+checkModel <- function(fittedModel, stop = F){
+  
+  out = T
+  
+  if(!(class(fittedModel)[1] %in% getPossibleModels())){
+    if(stop == FALSE) warning("DHARMa: fittedModel not in class of supported models. Absolutely no guarantee that this will work!")
+    else stop("DHARMa: fittedModel not in class of supported models") 
+  } 
+  
+  # if(hasWeigths(fittedModel)) warning("Your fitted model includes weights. For many GLMs, weights are not included in the simulations, and simulated quantile residuals are therefore not reliable. See ?checkModel for details")
+  
+  # if(hasNA(fittedModel)) message("It seems there were NA values in the data used for fitting the model. This can create problems if you supply additional data to DHARMa functions. See ?checkModel for details")
   
   if (class(fittedModel)[1] == "gam" ) if (class(fittedModel$family)[1] == "extended.family") stop("It seems you are trying to fit a model from mgcv that was fit with an extended.family. Simulation functions for these families are not yet implemented in DHARMa. See issue https://github.com/florianhartig/DHARMa/issues/11 for updates about this")
 
@@ -210,13 +195,31 @@ checkModel <- function(fittedModel){
 
 
 
-securityAssertion <- function(context = "Not provided", stop = F){
-  generalMessage = "Message from DHARMa package: a security assertion was not met. This means that during the execution of a DHARMa function, some unexpected conditions occurred. Even if you didn't get an error, your results may not be reliable. Please check with the help if you use the functions as intended. If you think that the error is not on your side, I would be grateful if you could report the problem at https://github.com/florianhartig/DHARMa/issues \n\n Context:"
-  if (stop == F) warning(paste(generalMessage, context))  
-  else stop(paste(generalMessage, context))  
+#' Check simulated data
+#' 
+#' The function checks if the simulated data seems fine
+#' 
+#' @param simulatedResponse the simulated response
+#' @param nObs number of observations
+#' @param nSim number of simulations
+#' 
+#' @keywords internal
+checkSimulations <- function(simulatedResponse, nObs, nSim){
+  
+  if(!inherits(simulatedResponse, "matrix")) securityAssertion("Simulation from the model produced wrong class", stop = T)
+  
+  if(any(dim(simulatedResponse) != c(nObs, nSim) )) securityAssertion("Simulation from the model produced wrong dimension", stop = T)
+  
+  if(any(!is.finite(simulatedResponse))) message("Simulations from your fitted model produce infinite values. Consider if this is sensible")
+  
+  if(any(is.nan(simulatedResponse))) securityAssertion("Simulations from your fitted model produce NaN values. DHARMa cannot calculated residuals for this. This is nearly certainly an error of the regression package you are using", stop = T)
+  if(any(is.na(simulatedResponse))) securityAssertion("Simulations from your fitted model produce NA values. DHARMa cannot calculated residuals for this. This is nearly certainly an error of the regression package you are using", stop = T)
+  
 }
 
 
+
+
 #' Recalculate residuals with grouping
 #' 
 #' The purpose of this function is to recalculate scaled residuals per group, based on the simulations done by \code{\link{simulateResiduals}}
@@ -224,12 +227,16 @@ securityAssertion <- function(context = "Not provided", stop = F){
 #' @param simulationOutput an object with simulated residuals created by \code{\link{simulateResiduals}}
 #' @param group group of each data point
 #' @param aggregateBy function for the aggregation. Default is sum. This should only be changed if you know what you are doing. Note in particular that the expected residual distribution might not be flat any more if you choose general functions, such as sd etc. 
+#' @param seed the random seed to be used within DHARMa. The default setting, recommended for most users, is keep the random seed on a fixed value 123. This means that you will always get the same randomization and thus teh same result when running the same code. NULL = no new seed is set, but previous random state will be restored after simulation. FALSE = no seed is set, and random state will not be restored. The latter two options are only recommended for simulation experiments. See vignette for details.
 #' 
 #' @return an object of class DHARMa, similar to what is returned by \code{\link{simulateResiduals}}, but with additional outputs for the new grouped calculations. Note that the relevant outputs are 2x in the object, the first is the grouped calculations (which is returned by $name access), and later another time, under identical name, the original output. Moreover, there is a function 'aggregateByGroup', which can be used to aggregate predictor variables in the same way as the variables calculated here 
 #' 
 #' @example inst/examples/simulateResidualsHelp.R
 #' @export
-recalculateResiduals <- function(simulationOutput, group = NULL, aggregateBy = sum){
+recalculateResiduals <- function(simulationOutput, group = NULL, aggregateBy = sum, seed = 123){
+  
+  randomState <-getRandomState(seed)
+  on.exit({randomState$restoreCurrent()})
 
   if(!is.null(simulationOutput$original)) simulationOutput = simulationOutput$original
 
@@ -244,10 +251,10 @@ recalculateResiduals <- function(simulationOutput, group = NULL, aggregateBy = s
   
   out$observedResponse = aggregateByGroup(simulationOutput$observedResponse)
   out$fittedPredictedResponse = aggregateByGroup(simulationOutput$fittedPredictedResponse)
-  out$simulatedResponse = apply(simulationOutput$simulatedResponse, 2, aggregateByGroup)
-  
+
   if (simulationOutput$refit == F){
     
+    out$simulatedResponse = apply(simulationOutput$simulatedResponse, 2, aggregateByGroup)
     out$scaledResiduals = getQuantile(simulations = out$simulatedResponse , observed = out$observedResponse , n = out$nGroups, nSim = simulationOutput$nSim, integerResponse = simulationOutput$integerResponse)
  
   ######## refit = T ##################   
@@ -267,6 +274,7 @@ recalculateResiduals <- function(simulationOutput, group = NULL, aggregateBy = s
   
   out$aggregateByGroup = aggregateByGroup
   out = c(out, simulationOutput)
+  out$randomState = randomState
   class(out) = "DHARMa"
   return(out)
 }
