@@ -14,16 +14,15 @@ runif(2)
 
 # if no seed is set, this will also be restored
 
-rm(.Random.seed)
-
+rm(.Random.seed) # now, there is no random seed
 x = getRandomState(123)
+exists(".Random.seed")  # TRUE
 runif(1)
 x$restoreCurrent()
-exists(".Random.seed")
+exists(".Random.seed") # False
 
-# with false 
+# with seed = false 
 
-rm(.Random.seed)
 x = getRandomState(seed = FALSE)
 exists(".Random.seed")
 runif(1)
